@@ -1,6 +1,6 @@
 package Test::Verbose;
 
-$VERSION = 0.007;
+$VERSION = 0.009;
 
 #BEGIN {
 #    *CORE::GLOBAL::chdir = \&my_chdir;
@@ -135,7 +135,7 @@ of Perl truth) value, otherwise a fatar error will be reported.
 
 Shortcut for
 
-    my $tv = Test::Verbose->new( %options )->exec_make_test( @names );
+    my $tv = Test::Verbose->new( %options )->test( @names );
 
 =cut
 
@@ -155,7 +155,7 @@ BEGIN {
 
 sub test_verbose {
     my $options = ref $_[-1] eq "HASH" ? pop : {};
-    Test::Verbose->new( %$options )->exec_make_test( @_ );
+    Test::Verbose->new( %$options )->test( @_ );
 }
 
 =head1 METHODS
@@ -197,7 +197,18 @@ of perl.
 
 sub new {
     my $proto = shift;
-    return bless { @_ }, ref $proto ? ref $proto : $proto;
+    my $self = bless { @_ }, ref $proto ? ref $proto : $proto;
+    $self->{TestPOD} = $self->{Compile} = $self->{RunTests} = 1
+        unless $self->{TestPOD} || $self->{Compile} || $self->{RunTests};
+
+    $self->{TestPOD}       = 0 if $self->{NoPOD};
+    $self->{Compile} = 0 if $self->{NoCompile};
+    $self->{RunTests}      = 0 if $self->{NoTests};
+
+    $self->{DoubleQuiet} ||= $self->{TripleQuiet};
+    $self->{Quiet}       ||= $self->{DoubleQuiet};
+
+    return $self;
 }
 
 =item dir
@@ -645,11 +656,11 @@ sub test_scripts_for_pod_file {
 #}
 
 
-=item exec_make_test
+=item test
 
-    $self->exec_make_test( @test_scripts );
+    $self->test( @test_scripts );
 
-chdir()s to C<$self->dir> and C<exec()>s make test.  Does not return.
+chdir()s to C<$self->dir> and C<exec()>s make test.
 
 =cut
 
@@ -665,7 +676,7 @@ sub _esc {
         @_;
 }
 
-sub exec_make_test {
+sub test {
     my $self = shift;
 
     my $cwd = Cwd::cwd;
@@ -673,6 +684,9 @@ sub exec_make_test {
     chdir $d or die "tv: $!: $d";
 
     my $rc_file = ".tvrc";
+
+    ## TODO: an option to name the log file.
+    open LOG, ">tv.log" unless $self->{NoLog};
     
     $rc_file = -e $rc_file
         ? do {
@@ -685,32 +699,32 @@ sub exec_make_test {
         }
         : undef;
 
-warn "[[$rc_file]]";
-
     eval $rc_file or die $@
        if defined $rc_file;
 
-    $self->{PodChecks} = [];
+    $self->{PodChecks}     = [];
     $self->{CompileChecks} = [];
-    $self->{Unhandled} = [];
+    $self->{Unhandled}     = [];
 
     my @scripts = @_ ? $self->test_scripts_for( @_ ) : ();
 
-    if ( ! $self->{NoPodChecker} && @{$self->{PodChecks}} ) {
+    if ( $self->{TestPOD} && @{$self->{PodChecks}} ) {
         ## NOTE: not using $^X here because podchecker may be from a
         ## newer perl.  Could lead to unexpected behavior, but very, very
         ## probably not.
         warn "tv\$ podchecker ", join( " ", _esc @{$self->{PodChecks}} ), "\n"
             if $self->{JustPrint} || $self->{Verbose};
+        ## TODO: log the output of this
         system "podchecker", @{$self->{PodChecks}}
             and die "tv: POD checks failed, not running further tests.\n"
             unless $self->{JustPrint};
     }
 
-    if ( ! $self->{NoCompileChecks} && @{$self->{CompileChecks}} ) {
+    if ( $self->{Compile} && @{$self->{CompileChecks}} ) {
         warn "tv\$ perl -Ilib -cw ", join( " ", _esc @{$self->{CompileChecks}} ), "\n"
             if $self->{JustPrint} || $self->{Verbose};
 
+        ## TODO: log the output of this
         system $^X, "-Ilib", "-cw", @{$self->{CompileChecks}}
             and die "tv: compile test failed, not running further tests.\n"
             unless $self->{JustPrint};
@@ -719,26 +733,29 @@ warn "[[$rc_file]]";
     $self->unhandled( @{$self->{Unhandled}} )
         if @{$self->{Unhandled}};
 
-    return unless @scripts;
+    return unless $self->{RunTests} && @scripts;
 
     my $debug = $self->{Debug} || $self->{DebugRun};
 
     my @cmds =
         $debug
-            ? map [ $^X, "-w", "-Ilib", "-d", $_ ], @scripts
-            : [
-                $self->{ExtUtils}
-                    ? (
-                        $^X,
-                        qw( -MExtUtils::Command::MM -e ),
-                        "test_harness(1,'lib')",
-                        @scripts
-                    )
-                    : ( qw( make test TEST_VERBOSE=1 ),
-                        @_
-                            ? "TEST_FILES=" . join " ", @scripts
-                            : (),
-                    )
+                ? (
+                    [ "make", "pm_to_blib" ],
+                    map [ $^X, "-w", "-Iblib/lib", "-d", $_ ], @scripts
+                )
+            : $self->{PurePerl}
+                ? map [ $^X, "-w", "-Ilib", $_ ], @scripts
+            : $self->{ExtUtils}
+                ? [
+                    $^X,
+                    qw( -MExtUtils::Command::MM -e ),
+                    "test_harness(1,'lib')",
+                    @scripts
+                ]
+                : [ qw( make test TEST_VERBOSE=1 ),
+                    @_
+                        ? "TEST_FILES=" . join " ", @scripts
+                        : (),
                 ];
 
     my $nonlazy_dyn_link = $self->{ExtUtils} || $debug;
@@ -758,6 +775,8 @@ tv: ** or rerun with -dd if you need to enter the debugger on startup. **
 
 TOHERE
 
+    my @result_codes;
+
     for ( @cmds ) {
         my $cmd = join " ", _esc @$_;
 
@@ -767,20 +786,75 @@ TOHERE
         warn "tv\$ $cmd\n"
             if $self->{JustPrint} || $self->{Verbose};
 
-        unless ( $self->{JustPrint} ) {
-            if ( @cmds > 1 ) {
-                system @$_ and die "tv: $!: $cmd\n";
+        next if $self->{JustPrint};
+
+        require IPC::Open3;
+
+        ## Collect both stderr and stdout in to one stream; don't
+        ## want select() and this lets the OS interleave output
+        ## better, I hope.
+        my $pid = IPC::Open3::open3( \*STDIN, my $out, undef, @$_ );
+        die "tv: $!: $cmd\n" unless defined $pid;
+
+        my @out;
+        my $saw_lone_not;
+        my $last_ok_line = "";
+        my $saw_ok;
+        while ( <$out> ) {
+            print LOG;
+            print unless $self->{Quiet};
+            push @out, $_ if $self->{Quiet};
+
+            $saw_lone_not = 1, next if /^not\s*$/;
+
+            my ( $not, $ok, $what, $number, $why ) =
+                /\A((?:not\s+)?)(ok\b)\s*(\d*)(?:#\s*(\S+))?/;
+
+            next unless $ok;
+
+            $saw_ok = 1;
+
+            if ( $ok
+                && ! $saw_lone_not
+                && ( ! $not || ( $why || "" ) =~ /TODO/i )
+            ) {
+                @out = ();
             }
-            else {
-                { exec @$_; }
-                warn "tv: $!: $cmd\n";
-                eval { kill -9, $$ };
-                exit 1;
+            elsif (
+                $not || ( $saw_lone_not && $ok )
+                && $self->{Quiet}
+                && ! $self->{DoubleQuiet}
+            ) {
+                if ( defined $cmd && @cmds > 1 ) {
+                    print $cmd, ":\n" unless $self->{Verbose};
+                    $cmd = undef;
+                }
+                print $saw_lone_not ? "not $_" : $_;
             }
+            $saw_lone_not = 0;
+            $last_ok_line = ( $saw_lone_not ? "not " : "" ) . $_ if $ok;
         }
+
+        ## Emit the next bit even in DoubleQuiet mode; it's usually
+        ## the All tests sucessful.  At least when I'm driving it ;).
+        if ( ! $self->{TripleQuiet} && @out ) {
+            if ( defined $cmd && @cmds > 1 ) {
+                print $cmd, ":\n" unless $self->{Verbose};
+                $cmd = undef;
+            }
+            print
+                $saw_ok ? "tv: after last 'ok/not ok':\n" : $last_ok_line,
+                @out;
+        }
+
+        waitpid $pid, 0 or warn "$! waiting on PID $pid";
+
+        push @result_codes, $? >> 8;
     }
 
-    exit 0;
+    close LOG;
+
+    (grep $_, @result_codes )[0] || 0;
 }
 
 =back
